import java.util.*;

public class MultiwayMergeIterator implements Iterator<Integer> {

  // The iterators that we are controlling.
  private Iterator<Integer>[] input;
  // This priority queue contains (at most) one element from each
  // input iterator. If it contains no element from iterator i,
  // then this iterator has no more elements. 
  private PriorityQueue<Integer> queue;

  public MultiwayMergeIterator (Iterator<Integer>[] input) {
    this.input = input;
    // Create the priority queue.
    this.queue = new PriorityQueue<Integer> (
      input.length
    );
    // Populate the queue with one element from each input stream (if
    // there is one; otherwise, not a problem).
    for (int i = 0; i < input.length; i++)
      drawFrom(i);
  }

  private void drawFrom (int i) {
    // Draw an element from iterator i and add it to the queue.
    if (input[i].hasNext())
        queue.add(input[i].next());
  }

  public E next () {
    // Take the pair that contains the least element of the queue.
    Integer i = queue.poll();
    if (i == null)
      throw new NoSuchElementException ();
    // Replace this element in the priority queue with a new element
    // drawn from the same input iterator (if there is one).
    drawFrom(i);
    // Return the element that we took out of the queue.
    return element;
  }

  public boolean hasNext () {
    return queue.size() > 0;
  }

}
