import java.util.*;

// Run this code with assertions enabled: java -ea TestInterval

public class TestInterval {

  private static void makeSureAssertionsAreEnabled () {
    try {
      assert(false);
      System.out.println("Vous devez donner l'option -ea dans VM Arguments!");
      System.exit(1);
    } catch (AssertionError e) {}
  }

  // Test the iterator [it] over the interval [beginning..end].
  // Call [hasNext] [k] times at each step.
  private static void testIntervalIterator (int beginning, int end, Iterator<Integer> it, int k) {
    for (int i = beginning; i < end; i++) {
      // [hasNext] should report that there is an element.
      for (int h = 0; h < k; h++)
        assert (it.hasNext());
      // [next] should produce the correct element.
      assert (it.next() == i);
    }
    // [hasNext] should report that there are no more elements.
    for (int h = 0; h < k; h++)
      assert (!it.hasNext());
    // [next] should throw an exception.
    try {
      it.next();
      assert(false);
    } catch (NoSuchElementException e) {}
  }

  // Test iteration over an [Interval] object.
  private static void testInterval (int beginning, int end, Iterable<Integer> interval) {
    // Iterate several times over this object.
    // Try calling [hasNext] 0, 1, or 2 times,
    // as a buggy version of [hasNext] could work
    // in some but not all scenarios.
    testIntervalIterator(beginning, end, interval.iterator(), 0);
    testIntervalIterator(beginning, end, interval.iterator(), 0);
    testIntervalIterator(beginning, end, interval.iterator(), 1);
    testIntervalIterator(beginning, end, interval.iterator(), 1);
    testIntervalIterator(beginning, end, interval.iterator(), 2);
    testIntervalIterator(beginning, end, interval.iterator(), 2);
  }

  // Create and test an [Interval] object.
  private static void testInterval (int beginning, int end) {
    testInterval(beginning, end, new Interval (beginning, end));
  }

  public static void main(String[] args) {
    makeSureAssertionsAreEnabled();

    // Test various intervals.
    System.out.print("Testing Interval:");
    testInterval(-10, 10);
    testInterval(0, 10);
    testInterval(-10, 0);
    testInterval(0, 0);
    testInterval(1, 0);
    testInterval(Integer.MIN_VALUE, Integer.MIN_VALUE + 10);
    testInterval(Integer.MAX_VALUE - 10, Integer.MAX_VALUE);
    testInterval(Integer.MAX_VALUE, Integer.MAX_VALUE);
    System.out.println(" success!");

  }

}
