import java.util.*;

// Run this code with assertions enabled: java -ea TestMergeIterator

public class TestMergeIterator {

  private static void makeSureAssertionsAreEnabled () {
    try {
      assert(false);
      System.out.println("Vous devez donner l'option -ea dans VM Arguments!");
      System.exit(1);
    } catch (AssertionError e) {}
  }

  // Array concatenation.
  public static <T> T[] concat(T[] a1, T[] a2) {
    T[] a = Arrays.copyOf(a1, a1.length + a2.length);
    System.arraycopy(a2, 0, a, a1.length, a2.length);
    return a;
  }

  // Testing [MergeIterator].
  // The test data is given by two integer arrays, which must be sorted.
  private static void testMerge (Integer[] a1, Integer[] a2) {
    // Create a [MergeIterator] object.
    Iterator<Integer> it1 = new MergeIterator (
      Arrays.asList(a1).iterator(),
      Arrays.asList(a2).iterator()
    );
    // Concatenate the arrays, sort the result array, and create an iterator on it.
    Integer[] a = concat(a1, a2);
    Arrays.sort(a);
    Iterator<Integer> it2 = Arrays.asList(a).iterator();
    // Compare the iterators.
    while (true) {
      boolean b1 = it1.hasNext();
      boolean b2 = it2.hasNext();
      assert (b1 == b2);
      if (!b1)
        return;
      int i1 = it1.next();
      int i2 = it2.next();
      assert (i1 == i2);
    }
  }

  public static void main(String[] args) {
    makeSureAssertionsAreEnabled();
    System.out.print("Testing MergeIterator:");
    // Arrays of different lengths.
    Integer[] a1 = {4,8,10};
    Integer[] a2 = {1,20};
    testMerge(a1, a2);
    testMerge(a2, a1);
    // Empty array.
    Integer[] b1 = {};
    Integer[] b2 = {1};
    testMerge(b1, b2);
    testMerge(b2, b1);
    testMerge(b1, b1);
    // Arrays of same length.
    Integer[] c1 = {2,4,6,8,10};
    Integer[] c2 = {1,3,5,7,9};
    testMerge(c1, c2);
    testMerge(c2, c1);
    testMerge(c1, c1);
    testMerge(c2, c2);
    // Random data, with lots of duplicates.
    Random r = new Random ();
    for (int n = 0; n < 100; n++) {
     Integer[] d1 = new Integer [1000];
     Integer[] d2 = new Integer [1000];
     for (int i = 0; i < 1000; i++) {
       d1[i] = r.nextInt(20);
       d2[i] = r.nextInt(20);
     }
     Arrays.sort(d1);
     Arrays.sort(d2);
     testMerge(d1, d2);
     testMerge(d2, d1);
    }
    System.out.println(" success!");
  }

}
